@@ -55,6 +55,7 @@
 #include "control.h"
 #include "sdpd.h"
 #include "glib-helper.h"
+#include "dbus-common.h"
 
 #define AVCTP_PSM 23
 
@@ -68,11 +69,15 @@
 #define AVCTP_PACKET_CONTINUE	2
 #define AVCTP_PACKET_END	3
 
+#define AVRCP_REJPACKET_LEN     0x01
+#define AVRCP_REJPACKET_CODE    0x00
+
 /* ctype entries */
 #define CTYPE_CONTROL		0x0
 #define CTYPE_STATUS		0x1
 #define CTYPE_ACCEPTED		0x9
 #define CTYPE_STABLE		0xC
+#define CTYPE_REJECT            0xA
 
 /* opcodes */
 #define OP_UNITINFO		0x30
@@ -417,12 +422,16 @@ static void avctp_unref(struct avctp *session)
 {
 	sessions = g_slist_remove(sessions, session);
 
-	if (session->state == AVCTP_STATE_CONNECTED)
-		g_dbus_emit_signal(session->dev->conn,
-						session->dev->path,
-						AUDIO_CONTROL_INTERFACE,
-						"Disconnected",
-						DBUS_TYPE_INVALID);
+	if (session->state == AVCTP_STATE_CONNECTED) {
+		gboolean value = FALSE;
+		g_dbus_emit_signal(session->dev->conn, session->dev->path,
+					AUDIO_CONTROL_INTERFACE,
+					"Disconnected", DBUS_TYPE_INVALID);
+		emit_property_changed(session->dev->conn, session->dev->path,
+					AUDIO_CONTROL_INTERFACE, "Connected",
+					DBUS_TYPE_BOOLEAN, &value);
+	}
+
 	if (session->sock >= 0)
 		close(session->sock);
 	if (session->io)
@@ -439,6 +448,7 @@ static void avctp_unref(struct avctp *session)
 	g_free(session);
 }
 
+
 static gboolean session_cb(GIOChannel *chan, GIOCondition cond,
 				gpointer data)
 {
@@ -502,7 +512,7 @@ static gboolean session_cb(GIOChannel *chan, GIOCondition cond,
 		ret = write(session->sock, buf, packet_size);
 	}
 
-	if (avctp->packet_type == AVCTP_PACKET_SINGLE &&
+	else if (avctp->packet_type == AVCTP_PACKET_SINGLE &&
 			avctp->cr == AVCTP_COMMAND &&
 			avctp->pid == htons(AV_REMOTE_SVCLASS_ID) &&
 			avrcp->code == CTYPE_STATUS &&
@@ -514,6 +524,18 @@ static gboolean session_cb(GIOChannel *chan, GIOCondition cond,
 				"OP_UNITINFO" : "OP_SUBUNITINFO");
 		ret = write(session->sock, buf, packet_size);
 	}
+	else if(avctp->packet_type == AVCTP_PACKET_SINGLE)   
+	{
+		debug("AVRCP : INVALID PDU-ID received");
+		avctp->cr = AVCTP_RESPONSE;
+		avctp->ipid = 1;
+		avrcp->code = CTYPE_REJECT;
+		buf[11] = 0x00;  
+		buf[12] = AVRCP_REJPACKET_LEN;
+		buf[13] = AVRCP_REJPACKET_CODE;  
+		
+		ret = write(session->sock, buf, packet_size);
+	}
 
 	return TRUE;
 
@@ -597,22 +619,30 @@ static void init_uinput(struct avctp *session)
 		debug("AVRCP: uinput initialized for %s", address);
 }
 
-static void avctp_connect_session(struct avctp *session)
+static gboolean avctp_connect_session(struct avctp *session)
 {
 	GIOChannel *io;
+	gboolean value;
+
+	session->dev = manager_find_device(&session->dst, NULL, FALSE);
+	if (!session->dev) {
+		error("Connecting audio device not known (SDP not completed)");
+		return FALSE;
+	}
 
 	session->state = AVCTP_STATE_CONNECTED;
-	session->dev = manager_find_device(&session->dst, NULL, FALSE);
-	if (!session->dev)
-		return;
 
 	session->dev->control->session = session;
 
 	init_uinput(session);
 
+	value = TRUE;
 	g_dbus_emit_signal(session->dev->conn, session->dev->path,
-					AUDIO_CONTROL_INTERFACE, "Connected",
-					DBUS_TYPE_INVALID);
+				AUDIO_CONTROL_INTERFACE, "Connected",
+				DBUS_TYPE_INVALID);
+	emit_property_changed(session->dev->conn, session->dev->path,
+				AUDIO_CONTROL_INTERFACE, "Connected",
+				DBUS_TYPE_BOOLEAN, &value);
 
 	if (session->io)
 		g_source_remove(session->io);
@@ -622,6 +652,8 @@ static void avctp_connect_session(struct avctp *session)
 			G_IO_IN | G_IO_ERR | G_IO_HUP | G_IO_NVAL,
 			(GIOFunc) session_cb, session);
 	g_io_channel_unref(io);
+
+	return TRUE;
 }
 
 static void auth_cb(DBusError *derr, void *user_data)
@@ -645,6 +677,7 @@ static void avctp_server_cb(GIOChannel *chan, int err, const bdaddr_t *src,
 	struct l2cap_options l2o;
 	struct avctp *session;
 	GIOCondition flags = G_IO_ERR | G_IO_HUP | G_IO_NVAL;
+	struct audio_device *dev;
 	char address[18];
 
 	if (err < 0) {
@@ -664,6 +697,15 @@ static void avctp_server_cb(GIOChannel *chan, int err, const bdaddr_t *src,
 		goto drop;
 	}
 
+	dev = manager_get_device(src, dst);
+	if (!dev) {
+		error("Unable to get audio device object for %s", address);
+		goto drop;
+	}
+
+	if (!dev->control)
+		dev->control = control_init(dev);
+
 	session->state = AVCTP_STATE_CONNECTING;
 	session->sock = g_io_channel_unix_get_fd(chan);
 
@@ -673,7 +715,6 @@ static void avctp_server_cb(GIOChannel *chan, int err, const bdaddr_t *src,
 		err = errno;
 		error("getsockopt(L2CAP_OPTIONS): %s (%d)", strerror(err),
 				err);
-		avctp_unref(session);
 		goto drop;
 	}
 
@@ -694,12 +735,14 @@ static void avctp_server_cb(GIOChannel *chan, int err, const bdaddr_t *src,
 	return;
 
 proceed:
-	avctp_connect_session(session);
+	if (!avctp_connect_session(session))
+		goto drop;
 
 	return;
 
 drop:
-	close(session->sock);
+	g_io_channel_close(chan);
+	avctp_unref(session);
 }
 
 static GIOChannel *avctp_server_socket(const bdaddr_t *src, gboolean master)
@@ -723,12 +766,22 @@ static GIOChannel *avctp_server_socket(const bdaddr_t *src, gboolean master)
 static void avctp_connect_cb(GIOChannel *chan, int err, const bdaddr_t *src,
 			const bdaddr_t *dst, gpointer data)
 {
-	struct avctp *session = data;
+	struct control *control = data;
+	struct avctp *session = control->session;
 	struct l2cap_options l2o;
 	socklen_t len;
 	int sk;
 	char address[18];
 
+	if (!session) {
+		debug("avctp_connect_cb: session removed while connecting");
+		g_io_channel_close(chan);
+		g_io_channel_unref(chan);
+		return;
+	}
+
+	ba2str(&session->dst, address);
+
 	if (err < 0) {
 		avctp_unref(session);
 		error("AVCTP connect(%s): %s (%d)", address, strerror(-err),
@@ -736,7 +789,6 @@ static void avctp_connect_cb(GIOChannel *chan, int err, const bdaddr_t *src,
 		return;
 	}
 
-	ba2str(&session->dst, address);
 	debug("AVCTP: connected to %s", address);
 
 	g_io_channel_set_close_on_unref(chan, FALSE);
@@ -747,6 +799,7 @@ static void avctp_connect_cb(GIOChannel *chan, int err, const bdaddr_t *src,
 	len = sizeof(l2o);
 	if (getsockopt(sk, SOL_L2CAP, L2CAP_OPTIONS, &l2o, &len) < 0) {
 		err = errno;
+		g_io_channel_unref(chan);
 		avctp_unref(session);
 		error("getsockopt(L2CAP_OPTIONS): %s (%d)", strerror(err),
 				err);
@@ -756,14 +809,15 @@ static void avctp_connect_cb(GIOChannel *chan, int err, const bdaddr_t *src,
 	init_uinput(session);
 
 	g_dbus_emit_signal(session->dev->conn, session->dev->path,
-					AUDIO_CONTROL_INTERFACE, "Connected",
-					DBUS_TYPE_INVALID);
+				AUDIO_CONTROL_INTERFACE, "Connected",
+				DBUS_TYPE_INVALID);
 
 	session->state = AVCTP_STATE_CONNECTED;
 	session->mtu = l2o.imtu;
 	session->io = g_io_add_watch(chan,
 				G_IO_IN | G_IO_ERR | G_IO_HUP | G_IO_NVAL,
 				(GIOFunc) session_cb, session);
+	g_io_channel_unref(chan);
 }
 
 gboolean avrcp_connect(struct audio_device *dev)
@@ -785,7 +839,7 @@ gboolean avrcp_connect(struct audio_device *dev)
 	session->state = AVCTP_STATE_CONNECTING;
 
 	err = bt_l2cap_connect(&dev->src, &dev->dst, AVCTP_PSM, 0,
-				avctp_connect_cb, session);
+				avctp_connect_cb, control);
 	if (err < 0) {
 		avctp_unref(session);
 		error("Connect failed. %s(%d)", strerror(-err), -err);
@@ -933,14 +987,46 @@ static DBusMessage *control_is_connected(DBusConnection *conn,
 	return reply;
 }
 
+static DBusMessage *control_get_properties(DBusConnection *conn,
+					DBusMessage *msg, void *data)
+{
+	struct audio_device *device = data;
+	DBusMessage *reply;
+	DBusMessageIter iter;
+	DBusMessageIter dict;
+	gboolean value;
+
+	reply = dbus_message_new_method_return(msg);
+	if (!reply)
+		return NULL;
+
+	dbus_message_iter_init_append(reply, &iter);
+
+	dbus_message_iter_open_container(&iter, DBUS_TYPE_ARRAY,
+			DBUS_DICT_ENTRY_BEGIN_CHAR_AS_STRING
+			DBUS_TYPE_STRING_AS_STRING DBUS_TYPE_VARIANT_AS_STRING
+			DBUS_DICT_ENTRY_END_CHAR_AS_STRING, &dict);
+
+	/* Connected */
+	value = (device->control->session != NULL);
+	dict_append_entry(&dict, "Connected", DBUS_TYPE_BOOLEAN, &value);
+
+	dbus_message_iter_close_container(&iter, &dict);
+
+	return reply;
+}
+
 static GDBusMethodTable control_methods[] = {
-	{ "IsConnected",	"",	"b",	control_is_connected },
+	{ "IsConnected",	"",	"b",	control_is_connected,
+						G_DBUS_METHOD_FLAG_DEPRECATED },
+	{ "GetProperties",	"",	"a{sv}",control_get_properties },
 	{ NULL, NULL, NULL, NULL }
 };
 
 static GDBusSignalTable control_signals[] = {
-	{ "Connected",			""	},
-	{ "Disconnected",		""	},
+	{ "Connected",			"",	G_DBUS_SIGNAL_FLAG_DEPRECATED},
+	{ "Disconnected",		"",	G_DBUS_SIGNAL_FLAG_DEPRECATED},
+	{ "PropertyChanged",		"sv"	},
 	{ NULL, NULL }
 };
 
