@@ -276,7 +276,8 @@ static void headset_setup_complete(struct audio_device *dev, void *user_data)
 
 	if (!headset_lock(dev, hs->lock)) {
 		error("Unable to lock headset");
-		goto failed;
+		debug("Ignoring this");
+		/* goto failed; */
 	}
 
 	memset(buf, 0, sizeof(buf));
@@ -919,8 +920,7 @@ static void handle_setconfiguration_req(struct unix_client *client,
 
 	client->access_mode = req->access_mode;
 
-	start_config(dev, client);
-
+	start_config(dev, client); 
 	return;
 
 failed:
