@@ -29,13 +29,9 @@
 
 #include <stdio.h>
 #include <errno.h>
-#include <ctype.h>
-#include <fcntl.h>
-#include <unistd.h>
 #include <stdlib.h>
-#include <string.h>
-#include <signal.h>
 #include <time.h>
+#include <unistd.h>
 #include <sys/time.h>
 #include <sys/param.h>
 #include <sys/ioctl.h>
@@ -44,15 +40,35 @@
 #include <bluetooth/bluetooth.h>
 #include <bluetooth/hci.h>
 #include <bluetooth/hci_lib.h>
+#include <bluetooth/sdp.h>
 
 #include <glib.h>
 
 #include <dbus/dbus.h>
 
 #include "hcid.h"
+#include "logging.h"
 #include "textfile.h"
+
 #include "adapter.h"
 #include "dbus-hci.h"
+#include "storage.h"
+
+typedef enum {
+	REQ_PENDING,
+	REQ_SENT
+} req_status_t;
+
+struct hci_req_data {
+	int dev_id;
+	int event;
+	req_status_t status;
+	bdaddr_t dba;
+	uint16_t ogf;
+	uint16_t ocf;
+	void *cparam;
+	int clen;
+};
 
 struct g_io_info {
 	GIOChannel	*channel;
@@ -64,7 +80,9 @@ static struct g_io_info io_data[HCI_MAX_DEV];
 
 static GSList *hci_req_queue = NULL;
 
-struct hci_req_data *hci_req_data_new(int dev_id, const bdaddr_t *dba, uint16_t ogf, uint16_t ocf, int event, const void *cparam, int clen)
+static struct hci_req_data *hci_req_data_new(int dev_id, const bdaddr_t *dba,
+					uint16_t ogf, uint16_t ocf, int event,
+					const void *cparam, int clen)
 {
 	struct hci_req_data *data;
 
@@ -121,7 +139,7 @@ static void hci_req_queue_process(int dev_id)
 	hci_close_dev(dd);
 }
 
-void hci_req_queue_append(struct hci_req_data *data)
+static void hci_req_queue_append(struct hci_req_data *data)
 {
 	GSList *l;
 	struct hci_req_data *match;
@@ -287,11 +305,12 @@ static void link_key_request(int dev, bdaddr_t *sba, bdaddr_t *dba)
 	bacpy(&req.bdaddr, dba);
 
 	err = ioctl(dev, HCIGETAUTHINFO, (unsigned long) &req);
-	if (err < 0 && errno != EINVAL)
-		debug("HCIGETAUTHINFO failed %s (%d)",
-					strerror(errno), errno);
-	else
+	if (err < 0) {
+		if (errno != EINVAL)
+			debug("HCIGETAUTHINFO failed %s (%d)",
+						strerror(errno), errno);
 		req.type = 0x00;
+	}
 
 	debug("kernel auth requirements = 0x%02x", req.type);
 
@@ -348,7 +367,7 @@ static void link_key_notify(int dev, bdaddr_t *sba, void *ptr)
 						DISCONNECT_CP_SIZE, &cp);
 		}
 	} else
-		hcid_dbus_bonding_process_complete(sba, dba, 0);
+		hcid_dbus_link_key_notify(sba, dba);
 
 	io_data[dev_id].pin_length = -1;
 }
@@ -587,7 +606,7 @@ static inline void remote_version_information(int dev, bdaddr_t *sba, void *ptr)
 	if (get_bdaddr(dev, sba, btohs(evt->handle), &dba) < 0)
 		return;
 
-	write_version_info(sba, &dba, btohs(evt->manufacturer),
+	write_version_info(sba, &dba, btohs(evt->err),
 				evt->lmp_ver, btohs(evt->lmp_subver));
 }
 
@@ -756,8 +775,7 @@ static inline void auth_complete(int dev, bdaddr_t *sba, void *ptr)
 	if (get_bdaddr(dev, sba, btohs(evt->handle), &dba) < 0)
 		return;
 
-	if (evt->status)
-		hcid_dbus_bonding_process_complete(sba, &dba, evt->status);
+	hcid_dbus_bonding_process_complete(sba, &dba, evt->status);
 }
 
 static inline void conn_request(int dev, bdaddr_t *sba, void *ptr)
@@ -771,6 +789,20 @@ static inline void conn_request(int dev, bdaddr_t *sba, void *ptr)
 	write_remote_class(sba, &evt->bdaddr, class);
 }
 
+static void delete_channel(GIOChannel *chan)
+{
+	int i;
+
+	/* Look for the GIOChannel in the table */
+	for (i = 0; i < HCI_MAX_DEV; i++)
+		if (io_data[i].channel == chan) {
+			stop_security_manager(i);
+			return;
+		}
+
+	error("IO channel not found in the io_data table");
+}
+
 static gboolean io_security_event(GIOChannel *chan, GIOCondition cond, gpointer data)
 {
 	unsigned char buf[HCI_MAX_EVENT_SIZE], *ptr = buf;
@@ -781,14 +813,14 @@ static gboolean io_security_event(GIOChannel *chan, GIOCondition cond, gpointer
 	GIOError err;
 
 	if (cond & (G_IO_NVAL | G_IO_HUP | G_IO_ERR)) {
-		g_io_channel_unref(chan);
+		delete_channel(chan);
 		return FALSE;
 	}
 
 	if ((err = g_io_channel_read(chan, (gchar *) buf, sizeof(buf), &len))) {
 		if (err == G_IO_ERROR_AGAIN)
 			return TRUE;
-		g_io_channel_unref(chan);
+		delete_channel(chan);
 		return FALSE;
 	}
 
