@@ -45,8 +45,11 @@
 #include <bluetooth/sdp_lib.h>
 
 #include "textfile.h"
+#include "glib-helper.h"
+#include "storage.h"
 
-static inline int create_filename(char *buf, size_t size, const bdaddr_t *bdaddr, const char *name)
+static inline int create_filename(char *buf, size_t size,
+				const bdaddr_t *bdaddr, const char *name)
 {
 	char addr[18];
 
@@ -400,12 +403,13 @@ int read_l2cap_info(bdaddr_t *local, bdaddr_t *peer,
 	return 0;
 }
 
-int write_version_info(bdaddr_t *local, bdaddr_t *peer, uint16_t manufacturer, uint8_t lmp_ver, uint16_t lmp_subver)
+int write_version_info(bdaddr_t *local, bdaddr_t *peer, uint16_t err,
+					uint8_t lmp_ver, uint16_t lmp_subver)
 {
 	char filename[PATH_MAX + 1], addr[18], str[16];
 
 	memset(str, 0, sizeof(str));
-	sprintf(str, "%d %d %d", manufacturer, lmp_ver, lmp_subver);
+	sprintf(str, "%d %d %d", err, lmp_ver, lmp_subver);
 
 	create_filename(filename, PATH_MAX, local, "manufacturers");
 
@@ -767,35 +771,45 @@ int store_record(const gchar *src, const gchar *dst, sdp_record_t *rec)
 	return err;
 }
 
-sdp_record_t *fetch_record(const gchar *src, const gchar *dst, const uint32_t handle)
+sdp_record_t *record_from_string(const gchar *str)
 {
-	char filename[PATH_MAX + 1], key[28], tmp[3], *str;
 	sdp_record_t *rec;
 	int size, i, len;
 	uint8_t *pdata;
+	char tmp[3];
+
+	size = strlen(str)/2;
+	pdata = g_malloc0(size);
+
+	tmp[2] = 0;
+	for (i = 0; i < size; i++) {
+		 memcpy(tmp, str + (i * 2), 2);
+		 pdata[i] = (uint8_t) strtol(tmp, NULL, 16);
+	}
+
+	rec = sdp_extract_pdu(pdata, size, &len);
+	free(pdata);
+
+	return rec;
+}
+
+
+sdp_record_t *fetch_record(const gchar *src, const gchar *dst,
+						const uint32_t handle)
+{
+	char filename[PATH_MAX + 1], key[28], *str;
+	sdp_record_t *rec;
 
 	create_name(filename, PATH_MAX, STORAGEDIR, src, "sdp");
 
 	snprintf(key, sizeof(key), "%17s#%08X", dst, handle);
 
 	str = textfile_get(filename, key);
-
 	if (!str)
 		return NULL;
 
-	size = strlen(str) / 2;
-	pdata = g_malloc0(size);
-
-	memset(tmp, 0, sizeof(tmp));
-	for (i = 0; i < size; i++) {
-		memcpy(tmp, str + (i*2), 2);
-		pdata[i] = (uint8_t) strtol(tmp, NULL, 16);
-	}
-
-	rec = sdp_extract_pdu(pdata, size, &len);
-
+	rec = record_from_string(str);
 	free(str);
-	free(pdata);
 
 	return rec;
 }
@@ -810,3 +824,207 @@ int delete_record(const gchar *src, const gchar *dst, const uint32_t handle)
 
 	return textfile_del(filename, key);
 }
+
+struct record_list {
+	sdp_list_t *recs;
+	const gchar *addr;
+};
+
+static void create_stored_records_from_keys(char *key, char *value,
+							void *user_data)
+{
+	struct record_list *rec_list = user_data;
+	const gchar *addr = rec_list->addr;
+	sdp_record_t *rec;
+
+	if (strncmp(key, addr, 17))
+		return;
+
+	rec = record_from_string(value);
+
+	rec_list->recs = sdp_list_append(rec_list->recs, rec);
+}
+
+sdp_list_t *read_records(bdaddr_t *src, bdaddr_t *dst)
+{
+	char filename[PATH_MAX + 1];
+	struct record_list rec_list;
+	char srcaddr[18], dstaddr[18];
+
+	ba2str(src, srcaddr);
+	ba2str(dst, dstaddr);
+
+	rec_list.addr = dstaddr;
+	rec_list.recs = NULL;
+
+	create_name(filename, PATH_MAX, STORAGEDIR, srcaddr, "sdp");
+	textfile_foreach(filename, create_stored_records_from_keys, &rec_list);
+
+	return rec_list.recs;
+}
+
+sdp_record_t *find_record_in_list(sdp_list_t *recs, const char *uuid)
+{
+	sdp_list_t *seq;
+
+	for (seq = recs; seq; seq = seq->next) {
+		sdp_record_t *rec = (sdp_record_t *) seq->data;
+		sdp_list_t *svcclass = NULL;
+		char *uuid_str;
+
+		if (sdp_get_service_classes(rec, &svcclass) < 0)
+			continue;
+
+		/* Extract the uuid */
+		uuid_str = bt_uuid2string(svcclass->data);
+		if (!uuid_str)
+			continue;
+
+		if (!strcasecmp(uuid_str, uuid)) {
+			sdp_list_free(svcclass, free);
+			free(uuid_str);
+			return rec;
+		}
+
+		sdp_list_free(svcclass, free);
+		free(uuid_str);
+	}
+	return NULL;
+}
+
+int store_device_id(const gchar *src, const gchar *dst,
+				const uint16_t source, const uint16_t vendor,
+				const uint16_t product, const uint16_t version)
+{
+	char filename[PATH_MAX + 1], str[20];
+
+	create_name(filename, PATH_MAX, STORAGEDIR, src, "did");
+
+	create_file(filename, S_IRUSR | S_IWUSR | S_IRGRP | S_IROTH);
+
+	snprintf(str, sizeof(str), "%04X %04X %04X %04X", source,
+						vendor, product, version);
+
+	return textfile_put(filename, dst, str);
+}
+
+static int read_device_id_from_did(const gchar *src, const gchar *dst,
+					uint16_t *source, uint16_t *vendor,
+					uint16_t *product, uint16_t *version)
+{
+	char filename[PATH_MAX + 1];
+	char *str, *vendor_str, *product_str, *version_str;
+
+	create_name(filename, PATH_MAX, STORAGEDIR, src, "did");
+
+	str = textfile_get(filename, dst);
+	if (!str)
+		return -ENOENT;
+
+	vendor_str = strchr(str, ' ');
+	if (!vendor_str) {
+		free(str);
+		return -ENOENT;
+	}
+	*(vendor_str++) = 0;
+
+	product_str = strchr(vendor_str, ' ');
+	if (!product_str) {
+		free(str);
+		return -ENOENT;
+	}
+	*(product_str++) = 0;
+
+	version_str = strchr(product_str, ' ');
+	if (!version_str) {
+		free(str);
+		return -ENOENT;
+	}
+	*(version_str++) = 0;
+
+	if (source)
+		*source = (uint16_t) strtol(str, NULL, 16);
+	if (vendor)
+		*vendor = (uint16_t) strtol(vendor_str, NULL, 16);
+	if (product)
+		*product = (uint16_t) strtol(product_str, NULL, 16);
+	if (version)
+		*version = (uint16_t) strtol(version_str, NULL, 16);
+
+	free(str);
+
+	return 0;
+}
+
+int read_device_id(const gchar *srcaddr, const gchar *dstaddr,
+					uint16_t *source, uint16_t *vendor,
+					uint16_t *product, uint16_t *version)
+{
+	uint16_t lsource, lvendor, lproduct, lversion;
+	sdp_list_t *recs;
+	sdp_record_t *rec;
+	bdaddr_t src, dst;
+	int err;
+
+	err = read_device_id_from_did(srcaddr, dstaddr, &lsource,
+						vendor, product, version);
+	if (!err) {
+		if (lsource == 0xffff)
+			err = -ENOENT;
+
+		return err;
+	}
+
+	str2ba(srcaddr, &src);
+	str2ba(dstaddr, &dst);
+
+	recs = read_records(&src, &dst);
+	rec = find_record_in_list(recs, PNP_UUID);
+
+	if (rec) {
+		sdp_data_t *pdlist;
+
+		pdlist = sdp_data_get(rec, SDP_ATTR_VENDOR_ID_SOURCE);
+		lsource = pdlist ? pdlist->val.uint16 : 0x0000;
+
+		pdlist = sdp_data_get(rec, SDP_ATTR_VENDOR_ID);
+		lvendor = pdlist ? pdlist->val.uint16 : 0x0000;
+
+		pdlist = sdp_data_get(rec, SDP_ATTR_PRODUCT_ID);
+		lproduct = pdlist ? pdlist->val.uint16 : 0x0000;
+
+		pdlist = sdp_data_get(rec, SDP_ATTR_VERSION);
+		lversion = pdlist ? pdlist->val.uint16 : 0x0000;
+
+		err = 0;
+	}
+
+	sdp_list_free(recs, (sdp_free_func_t)sdp_record_free);
+
+	if (err) {
+		/* FIXME: We should try EIR data if we have it, too */
+
+		/* If we don't have the data, we don't want to go
+		   through the above search every time. */
+		lsource = 0xffff;
+		lvendor = 0x0000;
+		lproduct = 0x0000;
+		lversion = 0x0000;
+	}
+
+	store_device_id(srcaddr, dstaddr, lsource, lvendor, lproduct, lversion);
+
+	if (err)
+		return err;
+
+	if (source)
+		*source = lsource;
+	if (vendor)
+		*vendor = lvendor;
+	if (product)
+		*product = lproduct;
+	if (version)
+		*version = lversion;
+
+	return 0;
+}
